/* -- $Id: BaseConverter.java,v 1.12 2003/11/06 10:04:37 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.OutputKeys;

import java.io.*;
import java.util.Enumeration;
import java.util.Properties;
import java.net.URL;

import com.aurigalogic.aurigadoc.logger.*;

/**
 * This class is the base converter for the AurigaDoc package. Its main 
 * purpose is to provide common menthods for all other converter classes. 
 * All other converters are derived from this class.
 * 
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.12 $ $Date: 2003/11/06 10:04:37 $
 */

public class BaseConverter implements ADConverter {

	/** Options collection */
	protected Properties parameters = new Properties();

	/** Properties collection */
	protected Properties props = new Properties();

	/** AURIGADOC HOME */
	protected String AURIGADOC_HOME = "../";

	/** Stylesheet for HTML output */
	protected static final String XML2HTML = "xml2html";

	/** Stylesheet for MULTI-HTML output */
	protected static final String XML2MHTML = "xml2mhtml";

	/** Stylesheet for DHTML main frame output */
	protected static final String XML2DHTMLFRAMESET = "dhtml/xml2main";

	/** Stylesheet for DHTML navigation frame output */
	protected static final String XML2DHTMLNAV = "dhtml/xml2nav";

	/** Stylesheet for PDF output */
	protected static final String XML2PDF = "xml2fo";

	/** Stylesheet for PS output */
	protected static final String XML2PS = "xml2fo";

	/** Stylesheet for RTF output */
	protected static final String XML2RTF = "xml2fo";

	/** Stylesheet for AWT output */
	protected static final String XML2AWT = "xml2fo";

	/** Stylesheet for FO output */
	protected static final String XML2FO = "xml2fo";

	/** Stylesheet for JHELP HS  output */
	protected static final String XML2JHELPHS = "jhelp/xml2hs";

	/** Stylesheet for JHELP TOC  output */
	protected static final String XML2JHELPTOC = "jhelp/xml2toc";

	/** Stylesheet for JHELP Map  output */
	protected static final String XML2JHELPMAP = "jhelp/xml2map";

	/** Stylesheet for CHM HHC  output */
	protected static final String XML2CHMHHC = "chm/xml2hhc";

	/** Stylesheet for CHM HHP  output */
	protected static final String XML2CHMHHP = "chm/xml2hhp";

	/** Stylesheet for unix man source output */
	protected static final String XML2MAN = "xml2man";

	/** Stylesheet for OHJ HELP HS  output */
	protected static final String XML2OHJHS = "ohj/xml2hs";

	/** Stylesheet for OHJ TOC  output */
	protected static final String XML2OHJTOC = "ohj/xml2toc";

	/** Stylesheet for OHJ Map  output */
	protected static final String XML2OHJMAP = "ohj/xml2map";

	/** Output Method for html */
	protected static final String OUTPUT_HTML = "html";
	
	/** Output Method for xml */
	protected static final String OUTPUT_XML = "xml";
	
	/** Output Method for text */
	protected static final String OUTPUT_TEXT = "text";
	
	/** The input file path */
	protected String inputFile;

	/** The output file/dir path */
	protected String outputFile;

	/** The logger */
	protected Logger logger;

	/** The transformer parameters colelction */
	Properties transformerParameters;

	/** Identifier for specifying the main xsl which generates html, fo, man, etc */
	public static final String XSL = "xsl";

	/**
	 * Default constructor.
	 */
	public BaseConverter() {
		loadProperties();	
		transformerParameters = new Properties();
	}

	/**
	 * Set the output file.
	 *
	 * @param outFile the output file/dir path.
	 */
	public void setOutputFile(String outFile) {
		this.outputFile = outFile;
	}

	/**
	 * Set the input file.
	 *
	 * @param inFile the input file path.
	 */
	public void setInputFile(String inFile) {
		this.inputFile = inFile;
	}

	/**
	 * Set the logger.
	 *
	 * @param logger the logger to be used for logging.
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
	}


	/**
	 * Set the home path.
	 *
	 * @param home the path of aurigadoc installation.
	 */
	public void setHome(String home) {
		this.AURIGADOC_HOME = home;
	}


	/** 
	 * Converts the file given by the inputFile 
	 * store the output in the file named by outputFile.
	 * 
	 * @param inputFile the path of the inputFile file.
	 * @param outFile the path of the output file.
	 * @throws Exception in case of an error.
	 */
	public void convert(String inputFile, String outFile) 
			throws Exception {
		setInputFile(inputFile);
		setOutputFile(outFile);
		convert();
	}

	
	/**
	 * Main method wich does the conversion. This method does not 
	 * do anything and if called throws an exception.
	 * @throws Exception if called.
	 */
	public void convert() throws Exception {
		throw new Exception("Illegal invocation. Use a specific converter");
	}


	/**
	 * Takes the xml source and write the output to the given stream by 
	 * doing an xsl transformation.
	 *
	 * @param inputStream the input stream
	 * @param outputStream the output stream
	 * @param xslFile the xsl file path
	 * @param outputMethod the output method(text,html,xml)
	 * @throws Exception in case of an error.
	 */
	protected void doTransform(InputStream inputStream, 
			OutputStream outputStream, String xslFile, 
			String outputMethod) throws Exception {
		long t1 = System.currentTimeMillis();
		logInfo("Applying XSL..");
		String transformerImpl = getProperty("javax.xml.transform.TransformerFactory");
		if (transformerImpl != null) {
			System.setProperty("javax.xml.transform.TransformerFactory", 
				transformerImpl);
		}

		// Instantiate a TransformerFactory
		TransformerFactory tFactory = 
			TransformerFactory.newInstance();
		
		// Use the TransformerFactory to process the stylesheet Source and 
		// generate a Transformer.
		StreamSource xslSource = new StreamSource(xslFile);
		Transformer transformer = tFactory.newTransformer (xslSource);
		transformer.setOutputProperty(OutputKeys.METHOD, outputMethod);

		// set the transformer parameters which will be avilable to xsl
		setTransformerParameters(transformer);
		
		// Use the Transformer to transform an XML Source and send the
		// output to a Result object.
		StreamSource xmlSource = new StreamSource(inputStream);
		StreamResult result = 
			new StreamResult(outputStream);
		transformer.transform (xmlSource, result);
		long t2 = System.currentTimeMillis();
		logInfo("XSL Transformation done in " + (t2 - t1) + " msecs.");
	}

	/**
	 * Copies all files form the srcDir to destDir.
	 *
	 * @param srcDir the source directory path
	 * @param destDir the destination directory path
	 * @throws Exception in case of an error.
	 */     
	protected void copyFiles(String srcDir, String destDir) throws Exception {
		File src = new File(srcDir);
		File[] files = src.listFiles();
		File dest = new File(destDir);
		for (int i = 0; i < files.length; i++) {
			if (!files[i].isDirectory()) {
				copyFile(files[i], dest);
			}	
		}
	}

	/**
	 * Copies a given file to the destDir.
	 */
	private void copyFile(File sourceFile, File destDir) throws IOException {  
		byte[] buffer = new byte[4096];
		if (!destDir.exists()) {
			destDir.mkdirs();
		}
		File destFile = new File(destDir, sourceFile.getName());
		FileInputStream in = new FileInputStream(sourceFile);  
		FileOutputStream out = new FileOutputStream(destFile);  
		int len;  
		while ((len = in.read(buffer)) != -1) {    
			out.write(buffer, 0, len);  
		}  
		out.flush();
		out.close();  
		in.close();
		in = null;
		out = null;
	}

	/**
	 * Loads properties from aurigadoc.properties file.
	 */
	private void loadProperties() {
		try {
			InputStream inputStream = 
				getClass().getResourceAsStream("/aurigadoc.properties");
			props.load(inputStream);
			setHome(props.getProperty("aurigadoc.home"));
		} catch (Exception e) {
			logError("Cannot load properties.", e);
		}	

	}
		
	/**
	 * Return the value of the property identified by key key.
	 *
	 * @param key the key for the property
	 * @return the property value
	 */
	public String getProperty(String key) {
		return props.getProperty(key);
	}

	/**
	 * Get the xsl path as a url string for the default xsl.
	 * @param xslName the xsl file name
	 * @return the xsl path as a url string.
	 * @throws Exception in case of an error.
	 */
	protected String getDefaultXSLUrl(String xslName) throws Exception {
		URL fileURL = 
			new File(AURIGADOC_HOME + "/xsl/" 
				+ xslName + ".xsl").toURL();
		return fileURL.toString();
	}

	/** 
	 * Log the msg as info using the logger set with setLogger.
	 * If no logger is set no messages are logged.
	 * @param msg the message to log.
	 */
	protected void logInfo(String msg) {
		if (logger != null) {
			logger.info(msg);
		}
	}

	/** 
	 * Log the msg as debug using the logger set with setLogger.
	 * If no logger is set no messages are logged.
	 * @param msg the message to log.
	 */
	protected void logDebug(String msg) {
		if (logger != null) {
			logger.debug(msg);
		}
	}

	/** 
	 * Log the msg as error using the logger set with setLogger.
	 * If no logger is set no messages are logged.
	 * @param msg the message to log.
	 * @param e the exception to log.
	 */
	protected void logError(String msg, Exception e) {
		if (logger != null) {
			logger.error(msg, e);
		}
	}
	
	/** 
	 * Log the msg as warning using the logger set with setLogger.
	 * If no logger is set no messages are logged.
	 * @param msg the message to log.
	 */
	protected void logWarning(String msg) {
		if (logger != null) {
			logger.warn(msg);
		}
	}
		/**
	 * Set additional parameters required for each converter.
	 * @param parameters the parameters collection
	 */ 
	public void setParameters(Properties parameters) {
		this.parameters = parameters;
	}

	/**
	 * Get the value of the parameter speecified by key.
	 *
	 * @param key the key of the option
	 * @return the option value
	 */
	protected String getParameter(String key) {
		if (parameters == null) {
			return "";
		} else {
			return parameters.getProperty(key);
		}
	}

	/**
	 * Sets the transformer parameters.
	 */
	private void setTransformerParameters(Transformer transformer) {
		Enumeration e = transformerParameters.propertyNames();
		String paramName;
		while (e.hasMoreElements()) {
			paramName = e.nextElement().toString();
			transformer.setParameter(paramName, 
				transformerParameters.getProperty(paramName));
		}
	}

	/**
	 * Sets the tarnsformer parameter.
	 * 
	 * @param paramName the parameter name
	 * @param paramValue the parameter value
	 */
	public void setTransformerParameter(String paramName, String paramValue) {
		transformerParameters.setProperty(paramName, paramValue);
	}

	/**
	 * Gets the url of the xsl with the specified key.
	 * If an xsl with that key does not exist, the default
	 * specified is used.
	 *
	 * @param key the xsl with the given key
	 * @param defaultXSL the default xsl to use if the custom xsl is
	 * not specified.
	 *
	 * @return the xsl as a url string.
	 * @throws Exception in case of an error.
	 */
	protected String getXSLForKey(String key, String defaultXSL) 
			throws Exception {
		String xslFile = parameters.getProperty(key);
		String xslUrl = null;
		if (xslFile == null || xslFile.equals("")) {
			// we use the default one
			xslUrl = getDefaultXSLUrl(defaultXSL);
		} else {
			URL fileURL = new File(xslFile).toURL();
			xslUrl = fileURL.toString();
		}
		return xslUrl;
	}
}
